import { createStore, combineReducers, applyMiddleware } from 'redux'
import thunk from 'redux-thunk'
import fetch from 'isomorphic-fetch'

// Action Creator
function requestPosts() {
  return { type: 'REQUEST_POSTS' }
}

function receivePosts(json) {
  return {
    type: 'RECEIVE_POSTS',
    posts: json.map((post) => post.title)
  }
}

function fetchPosts() {
  return function(dispatch) {
    dispatch(requestPosts())
    return fetch('https://qiita.com/api/v2/items?query=tag:redux')
      .then(response => response.json())
      .then(json => dispatch(receivePosts(json)))
  }
}

// Reducer
function requestStatus(state = { isFetching: false }, action) {
  switch(action.type) {  
  case 'REQUEST_POSTS':
    return { isFetching: true }
  case 'RECEIVE_POSTS':
    return { isFetching: false }
  default:
    return state
  }
}

function items(state = [], action) {
  switch(action.type) {
  case 'RECEIVE_POSTS':
    return action.posts
  default:
    return state
  }
}

const reducer = combineReducers({ requestStatus, items })
const store = createStore(reducer, applyMiddleware(thunk))
store.subscribe(() => console.log(store.getState()))
store.dispatch(fetchPosts())

