import { createStore } from 'redux';
import { List, Map } from 'immutable';

const initialState = Map({
  visibilityFilter: 'SHOW_ALL',
  todos: List([
    Map({
      text: 'Consider using Rexux',
      completed: true
    }),
    Map({
      text: 'keep all state in a single tree',
      completed: false
    })
  ])
});

// Reducer
// state と action を受けて新しい state を返す
// Object.assign ... http://js-next.hatenablog.com/entry/2015/05/07/211859
function todoApp(state = initialState, action) {
  return Map({
    visibilityFilter: visibilityFilter(state.get("visibilityFilter"), action),
    todos: todos(state.get("todos"), action)
  });
}

function visibilityFilter(state = 'SHOW_ALL', action) {
  switch (action.type) {
    case 'SET_VISIBILITY_FILTER':
      return action.filter;
    default:
      return state;
  }
}

function todos(state = List(), action) {
  switch (action.type) {
    case 'ADD_TODO':
      return state.push(Map({text: action.text, completed: false }));
//      return //[
//        ...state,
//        {
//          text: action.text,
//          completed: false
//        }
//      ];
    case 'COMPLETE_TODO':
      return state.update(action.index, (todo) => { 
        return todo.update("completed", () => { return true; }); 
      });
//      return [
//        ...state.slice(0, action.index),
//        Object.assign({}, state[action.index], { completed: true }),
//        ...state.slice(action.index + 1)
//      ];
    default:
      return state;
  }
}

let store = createStore(todoApp);

store.subscribe(() => console.log(store.getState().toJS()));

store.dispatch({ type: 'SET_VISIBILITY_FILTER',  filter: 'SHOW_ALL' });
store.dispatch({ type: 'ADD_TODO',  text: 'Web+DB writing' });
store.dispatch({ type: 'SET_VISIBILITY_FILTER',  filter: 'SHOW_COMPLETED' });
store.dispatch({ type: 'COMPLETE_TODO',  index: 1 });
