import { createStore, combineReducers } from 'redux';

const initialState = {
  visibilityFilter: 'SHOW_ALL',
  todos: [
    {
      text: 'Consider using Rexux',
      completed: true
    },
    {
      text: 'keep all state in a single tree',
      completed: false
    }
  ]
};

// Reducer
// state と action を受けて新しい state を返す
// Object.assign ... http://js-next.hatenablog.com/entry/2015/05/07/211859

// function todoApp(state = initialState, action) {
//  return {
//    visibilityFilter: visibilityFilter(state.visibilityFilter, action),
//    todos: todos(state.todos, action)
//  };
//}

function visibilityFilter(state = 'SHOW_ALL', action) {
  switch (action.type) {
    case 'SET_VISIBILITY_FILTER':
      return action.filter;
    default:
      return state;
  }
}

function todos(state = [], action) {
  switch (action.type) {
    case 'ADD_TODO':
      return [
        ...state,
        {
          text: action.text,
          completed: false
        }
      ];
    case 'COMPLETE_TODO':
      return [
        ...state.slice(0, action.index),
        Object.assign({}, state[action.index], { completed: true }),
        ...state.slice(action.index + 1)
      ];
    default:
      return state;
  }
}

const todoApp = combineReducers({ visibilityFilter, todos });
let store = createStore(todoApp, initialState);

store.subscribe(() => console.log(store.getState()));

store.dispatch({ type: 'SET_VISIBILITY_FILTER',  filter: 'SHOW_ALL' });
store.dispatch({ type: 'ADD_TODO',  text: 'Web+DB writing' });
store.dispatch({ type: 'SET_VISIBILITY_FILTER',  filter: 'SHOW_COMPLETED' });
store.dispatch({ type: 'COMPLETE_TODO',  index: 1 });
